#------------------------------------------------------------------------------
#
#   Interesting Times - Editing Screen
#
#------------------------------------------------------------------------------

from pygame import Rect, Color
from albow.widget import Widget
from albow.layout import Row, Column, Frame
from albow.utils import frame_rect
from humerus.edit_commands import EditCommandScreen
from humerus.change_dot import ChangeDot
from tile_array_view import TileArrayView
from city_view import CityView
from editor import editor
from game import game
import palettes
import constants as k

class TilePaletteView(TileArrayView):

	sel_color = Color("blue")
	sel_thickness = 2

	def __init__(self, w, h):
		TileArrayView.__init__(self, size = (w * k.CELL_WIDTH, h * k.CELL_HEIGHT))
	
	def get_tile_array(self):
		return editor.selected_palette
	
	def draw_tile(self, surf, tile, pos):
		TileArrayView.draw_tile(self, surf, tile, pos)
		ta = self.get_tile_array()
		if tile is ta.selected_tile:
			r = tile.get_rect()
			r.topleft = pos
			frame_rect(surf, self.sel_color, r, self.sel_thickness)

	def click_cell(self, e, i, j):
		editor.selected_palette.select_cell(i, j)

#------------------------------------------------------------------------------

class LevelEditView(CityView):

	level = property(lambda self: game.level)
	
	def draw(self, surf):
		ta = self.get_tile_array()
		cells = ta.cells
		for i in ta.irange:
			for j in ta.jrange:
				tile = cells[i, j]
				if tile:
					i0, j0 = tile.origin
					x0 = i0 * k.CELL_WIDTH
					y0 = j0 * k.CELL_HEIGHT
					x = i * k.CELL_WIDTH
					y = j * k.CELL_HEIGHT
					surf.set_clip(x, y, k.CELL_WIDTH, k.CELL_HEIGHT)
					tile.draw(surf, (x0, y0))
		surf.set_clip(None)

	def click_cell(self, e, i, j):
		tile = editor.selected_palette.selected_tile
		if tile:
			self.tile_array.add_tile(tile.clone(), (i, j))
		else:
			self.tile_array.remove_tile_at((i, j))
		game.level_changed()

#------------------------------------------------------------------------------

class ITEditScreen(EditCommandScreen):

	def __init__(self, shell):
		EditCommandScreen.__init__(self, shell)
		level_view = LevelEditView(width = self.width)
		controls = self.make_edit_controls()
		change = ChangeDot(game, frame_width = 0)
		contents = Column([level_view, controls],
			height = self.height,
			expand = level_view,
			align = 'l',
			spacing = 0,
			)
		self.add(contents)
		change.bottomright = self.size
		self.add(change)

	def make_edit_controls(self):
		tile_palette_view = TilePaletteView(50, 4)
		tile_palette_view.palette = palettes.palette
		row = Row([tile_palette_view], align = 't')
		result = Frame(row, margin = 4)
		result.height = 120
		return result
