#------------------------------------------------------------------------------
#
#   Interesting Times - House tile
#
#------------------------------------------------------------------------------

from __future__ import division
from numpy import array, argmax
from pygame import Rect, Color
from albow.resource import get_image
from vectors import vector
from probability import chance
from tile import StaticTile, get_tile_image_array
import constants as k

class HouseTile(StaticTile):

	is_house = True
	newspaper_image = get_image("newspaper-delivered.png")
	newspaper_offset = vector(35, 39)
#	loyalty_bar_width = 56
#	loyalty_bar_height = 4
#	loyalty_bar_offset = vector(4, 56)
	bar_area = Rect(4, 56, 56, 4)
	#loyalty_bar_bg_color = Color("gray")

	newsstand = None
	subscription = 0
	newspaper_delivered = False
	loyalties = None
	
	loyalty = property(lambda self: lambda s: self.loyalties[s - 1])
	
	def prepare(self, level):
		#print self, "prepare" ###
		self.level = level
		self.loyalties = array([0.0, 0.0])
		self.neighbourly_influence = array([0.0, 0.0])
		neighbours = set()
		#cells = level.city.cells
		city = level.city
		i0, j0 = self.origin
		for i in xrange(i0 - 3, i0 + self.width + 3):
			for j in xrange(j0 - 3, j0 + self.height + 3):
				#tile = cells[i, j]
				tile = city.tile_at((i, j))
				if tile and tile is not self:
					#print "... neighbour =", tile ###
					if tile.is_house:
						neighbours.add(tile)
					elif tile.is_newsstand:
						self.newsstand = tile
		self.neighbours = neighbours
		#print "HouseTile.prepare:", self, "has neighbours", [
		#	str(h) for h in self.neighbours] ###
	
	def get_image(self):
		return get_tile_image_array(self.image_name, 3)[self.subscription]

	def draw(self, surf, pos):
		StaticTile.draw(self, surf, pos)
		if self.newspaper_delivered:
			surf.blit(self.newspaper_image, pos + self.newspaper_offset)
		self.draw_loyalty_bar(surf, pos)
	
	def draw_loyalty_bar(self, surf, pos):
		if self.loyalties is not None:
			s = self.subscription
			if 1: #s:
				l = self.influenced_loyalties()
				f = l / k.MAX_LOYALTY
				c = self.bar_colors
				for i in (0, 1):
					self.draw_bar(surf, pos, i, 2, f[i], c[i])
	
	def deliver(self, s, news_value, by_runner):
		self.loyalties[s - 1] += news_value * k.LOYALTY_PER_COLUMN_INCH
		#print "HouseTile.deliver: loyalties now", self.loyalties ###
		if by_runner:
			self.newspaper_delivered = True
	
	def update(self):
		self.newspaper_delivered = False
		self.calc_neighbourly_influence()
		if self.subscription:
			self.consider_unsubscribing()
			if self.subscription:
				self.consider_switching_subscription()
		else:
			self.consider_subscribing()
	
	def decay_loyalties(self):
		self.loyalties *= k.LOYALTY_DECAY_RATE
	
	def influenced_loyalties(self):
		return self.loyalties + self.neighbourly_influence
	
	def consider_unsubscribing(self):
		loyalty = self.loyalty(self.subscription)
		t = k.LOYALTY_UNSUBSCRIPTION_THRESHOLD
		#print self, "consider_unsubscribing:", loyalty, "<", t, "?" ###
		if loyalty < t:
			#if chance(1.0 - loyalty / t):
				self.subscription = k.NO_TEAM
	
	def consider_subscribing(self):
		#l = self.loyalties + self.neighbourly_influence
		l = self.influenced_loyalties()
		i = argmax(l)
		if self.subscription_chance(l[i]):
			self.subscription = i + 1
			#self.loyalties[i] = l[i]
	
	def consider_switching_subscription(self):
		#l = self.loyalties + self.neighbourly_influence
		l = self.influenced_loyalties()
		i = argmax(l)
		d = l[i] - l[1 - i]
		if self.subscription_chance(d):
			self.subscription = i + 1
			self.loyalties[i] = l[i]

	def subscription_chance(self, l):
		t = k.LOYALTY_SUBSCRIPTION_THRESHOLD
		return l > t
#		if l > t:
#			p = (l - t) / (k.MAX_LOYALTY - t)
#			return chance(p)
	
	def calc_neighbourly_influence(self):
		f = k.NEIGHBOURLY_INFLUENCE_FACTOR
		r = k.NEIGHBOURLY_INFLUENCE_FALLOFF
		l = array([0, 0])
		for h in self.neighbours:
			#print "neighbourly_influence:", h ###
			l += h.loyalties * f
			f *= r
		self.neighbourly_influence = l

	def purchasing(self):
		s = self.subscription
		if s:
			#print self, "paying subscription price to team", s ###
			self.level.team(s).receive(k.SUBSCRIPTION_PRICE)
		else:
			self.consider_newsstand_purchase()
	
	def consider_newsstand_purchase(self):
		#print "HouseTile.consider_newsstand_purchase:", self ###
		ns = self.newsstand
		if ns:
			nv = ns.news_value
			#print self, "considering newsstand purchase from", nv ###
			nvt = nv[0] + nv[1]
			if nvt > 0:
				p = nv[0] / nvt
				if chance(p):
					s = k.BLUE_TEAM
				else:
					s = k.RED_TEAM
				#print self, "paying newsstand price to team", s ###
				self.level.team(s).receive(k.NEWSSTAND_PRICE)
				#l = nv[s - 1] * k.LOYALTY_PER_COLUMN_INCH
				#self.consider_subscribing_under_influence(s, l)
				self.deliver(s, nv[s - 1], by_runner = False)

