#------------------------------------------------------------------------------
#
#   Interesting Times - Tile palettes
#
#------------------------------------------------------------------------------

from level import TileArray
from tiles import StaticTile, RoadTile, HouseTile, NewsStandTile

class TilePalette(TileArray):

	selected_tile = None

	def __init__(self, template, definitions):
		w = len(template[0])
		h = len(template)
		TileArray.__init__(self, w, h)
		for y, trow in enumerate(template):
			for x, tchar in enumerate(trow):
				tile = definitions.get(tchar)
				if tile:
					self.add_tile(tile, (x, y))
	
	def select_cell(self, i, j):
		if 0 <= i < self.cells_wide and 0 <= j < self.cells_high:
			self.selected_tile = self.cells[i, j]

#------------------------------------------------------------------------------

palette = TilePalette([
		"AU.B...C...D.E.F.G.H.I.P...Q...R...S...T...",
		"V..........................................",
		".W.........J.K.L.M.N.O.....................",
		"...........................................",
	], {
		"A": StaticTile(1, 1, "grass-1x1.png"),
		"B": StaticTile(4, 4, "grass-4x4.png"),
		"C": HouseTile(4, 4, "house-4x4.png"),
		"D": RoadTile(2, 2, "road-ns-2x2.png"),
		"E": RoadTile(2, 2, "road-corner-se-2x2.png"),
		"F": RoadTile(2, 2, "road-corner-sw-2x2.png"),
		"G": RoadTile(2, 2, "road-tee-e-2x2.png"),
		"H": RoadTile(2, 2, "road-tee-s-2x2.png"),
		"I": RoadTile(2, 2, "road-cross-2x2.png"),
		"J": RoadTile(2, 2, "road-ew-2x2.png"),
		"K": RoadTile(2, 2, "road-corner-ne-2x2.png"),
		"L": RoadTile(2, 2, "road-corner-nw-2x2.png"),
		"M": RoadTile(2, 2, "road-tee-n-2x2.png"),
		"N": RoadTile(2, 2, "road-tee-w-2x2.png"),
		"O": StaticTile(2, 2, "grass-2x2.png"),
		"P": RoadTile(4, 4, "road-end-e-4x4.png"),
		"Q": RoadTile(4, 4, "road-end-w-4x4.png"),
		"R": RoadTile(4, 4, "road-end-s-4x4.png"),
		"S": RoadTile(4, 4, "road-end-n-4x4.png"),
		"T": NewsStandTile(4, 4, "newsstand-4x4.png"),
		"U": StaticTile(2, 2, "grass-2x2.png"),
		"V": RoadTile(1, 2, "road-ew-1x2.png"),
		"W": RoadTile(2, 1, "road-ns-2x1.png"),
	})
