#------------------------------------------------------------------------------
#
#   Interesting Times - Team
#
#------------------------------------------------------------------------------

from runner import Runner
from reporter import Reporter
from sound import bleep, bling
import constants as k

class Team(object):
	#  level           Level
	#  team_no         int
	#  news_collected  int        to be published in next edition
	#  news_published  int        in current edition
	#  cash            number
	#  copies          int        newspapers remaining
	#  auto_play       boolean    is a computer player
	#  actors          [Actor]
	#  runners         [Runner]
	#  reporters       [Reporter]
	#  runner_start_cells  [cell]   candidate runner start positions
	#  next_runner_start_index  int   index into runner_start_cells
	
	auto_play = False
	copies_to_print = 20 ###
	
	city = property(lambda self: self.level.city)
	
	def __init__(self, n, level):
		self.level = level
		self.team_no = n
		self.news_collected = 0
		self.news_published = 0
		self.cash = k.INITIAL_CASH
		self.copies = 0
		self.actors = []
		self.runners = []
		self.reporters = []
		self.runner_start_cells = level.find_runner_start_cells()
		self.next_runner_start_index = 0
		#print "Team", self.team_no, "candidate runner start cells =", self.runner_start_cells ###
		self.hire_reporter()
		self.hire_runner()

	city = property(lambda self: self.level.city)
	
	def __str__(self):
		return "Team(%d)" % self.team_no
	
	def pick_runner_start_tile(self):
		i = self.next_runner_start_index
		cells = self.runner_start_cells
		result = cells[i]
		i += 1
		if i >= len(cells):
			i = 0
		self.next_runner_start_index = i
		return result

	def hire_runner(self):
		print self, "hiring runner" ###
		cell = self.pick_runner_start_tile() \
			or self.random_actor_position()
		runner = Runner(self, cell)
		self.add_runner(runner)
		return runner
		
	def hire_reporter(self):
		reporter = Reporter(self, self.random_actor_position())
		self.add_reporter(reporter)
		return reporter
	
	def dismiss_staff(self, actor):
		if actor in self.actors:
			self.actors.remove(actor)
		if actor in self.reporters:
			self.reporters.remove(actor)
		if actor in self.runners:
			self.runners.remove(actor)
		if actor is self.level.selected_actor:
			self.level.selected_actor = None
	
	def dismiss_last_runner(self):
		print self, "dismissing a runner" ###
		self.dismiss_staff(self.runners[-1])
	
	def can_hire_reporter(self):
		return len(self.reporters) < k.MAX_REPORTERS
	
	def can_hire_runner(self):
		return len(self.runners) < k.MAX_RUNNERS
		
	def can_dismiss_runner(self):
		return len(self.runners) > 1
	
	def can_dismiss_staff(self, actor):
		return self.can_dismiss_from(actor, self.reporters) \
			or self.can_dismiss_from(actor, self.runners)
	
	def can_dismiss_from(self, actor, staff):
		return actor in staff and len(staff) > 1
	
	def random_actor_position(self):
		road = self.level.random_road_tile()
		if road:
			cell = road.random_cell()
		else:
			cell = self.city.random_cell()
		return cell
		
	def add_runner(self, runner):
		self.runners.append(runner)
		self.actors.append(runner)

	def add_reporter(self, reporter):
		self.reporters.append(reporter)
		self.actors.append(reporter)

	def begin_frame(self):
		for actor in self.actors:
			actor.begin_frame()

	def print_newspaper(self):
		cost = self.copies_to_print * k.PRINTING_COST
		if self.pay(cost, "print newspapers"):
			news = self.news_collected
			self.news_published = news
			self.news_collected = 0
			self.copies = self.copies_to_print
			print self, "printed", self.copies, "copies" ###
			i = self.team_no - 1
			for newsstand in self.level.newsstands:
				newsstand.news_value[i] = news
			return True
		else:
			self.copies = 0
			return False

	def start_delivery(self):
		news = self.news_published
		#print "Team.start_delivery:", self.team_no, "with", news, "news" ###
		#if news:
		for runner in self.runners:
			runner.begin_day(news)
	
	def stop_delivery(self):
		for runner in self.runners:
			runner.go_off_duty()
	
	def receive(self, amount):
		self.cash += amount
	
	def pay(self, amount, desc):
		if self.cash >= amount:
			self.cash -= amount
			return True
		else:
			if self.team_no == k.BLUE_TEAM:
				bleep("Not enough cash to %s!" % desc)
			else:
				bling("The Ixian Bugle has gone broke!")
				self.level.completed = True
			return False

	def collect_news(self, news_value):
		self.news_collected += news_value
		#print "Team", self.team_no, "now has", self.news_collected, "news" ###

	def copies_available(self):
		return self.copies > 0
	
	def get_copy(self):
		if self.copies > 0:
			self.copies -= 1
			return True
		else:
			return False

	def pay_reporters(self):
		self.pay_staff(self.reporters, k.REPORTER_WAGE, "reporters")
	
	def pay_runners(self):
		self.pay_staff(self.runners, k.RUNNER_WAGE, "runners")
	
	def pay_staff(self, staff, wage, desc):
		pay_desc = "pay " + desc
		for actor in staff:
			actor.paid = False
		for actor in staff:
			if not self.pay(wage, pay_desc):
				return
			actor.paid = True

	def count_subscribers(self):
		s = self.team_no
		n = 0
		for h in self.level.houses:
			if h.subscription == s:
				n += 1
		print self, "has", n, "subscribers" ###
		return n

	def think(self):
		print self, "thinking" ###
		nsub = self.count_subscribers()
		self.copies_to_print = nsub + 10
		rsub = len(self.runners) * 10
		if nsub > rsub:
			if self.can_hire_runner():
				self.hire_runner()
		elif nsub < rsub - 5:
			if self.can_dismiss_runner():
				self.dismiss_last_runner()

